@@ -103,7 +103,6 @@ groupwise_store_construct (CamelService *service, CamelSession *session,
 				     _("Host or user not available in url"));
 	}
 
-	//store->flags = 0; 
 	groupwise_store->list_loaded = FALSE;
 	
 	/*storage path*/
@@ -170,9 +169,9 @@ groupwise_compare_folder_name (gconstpointer a, gconstpointer b)
 static gboolean
 groupwise_auth_loop (CamelService *service, CamelException *ex)
 {
-	CamelGroupwiseStore *groupwise_store = CAMEL_GROUPWISE_STORE (service);
 	CamelSession *session = camel_service_get_session (service);
 	CamelStore *store = CAMEL_STORE (service);
+	CamelGroupwiseStore *groupwise_store = CAMEL_GROUPWISE_STORE (store);
 	CamelGroupwiseStorePrivate *priv = groupwise_store->priv;
 	char *errbuf = NULL;
 	gboolean authenticated = FALSE;
@@ -184,6 +183,7 @@ groupwise_auth_loop (CamelService *service, CamelException *ex)
 	else 
 		uri = g_strconcat ("http://", priv->server_name, ":", priv->port, "/soap", NULL);
 	service->url->passwd = NULL;
+	
 
 	while (!authenticated) {
 		if (errbuf) {
@@ -193,6 +193,7 @@ groupwise_auth_loop (CamelService *service, CamelException *ex)
 			service->url->passwd = NULL;
 		}
 		
+	
 		if (!service->url->passwd && !(store->flags & CAMEL_STORE_PROXY)) {
 			char *prompt;
 			
@@ -215,8 +216,6 @@ groupwise_auth_loop (CamelService *service, CamelException *ex)
 			}
 		}
 		
-		
-				
 		priv->cnc = e_gw_connection_new (uri, priv->user, service->url->passwd);
 		if (!E_IS_GW_CONNECTION(priv->cnc) && priv->use_ssl && g_str_equal (priv->use_ssl, "when-possible")) {
 			char *http_uri = g_strconcat ("http://", uri + 8, NULL);
@@ -225,7 +224,7 @@ groupwise_auth_loop (CamelService *service, CamelException *ex)
 		}
 		if (!E_IS_GW_CONNECTION(priv->cnc)) {
 			errbuf = g_strdup_printf (_("Unable to authenticate "
-					    "to GroupWise server."));
+					    "to GroupWise server. "));
 						  
 			camel_exception_clear (ex);
 		} else 
@@ -270,10 +269,18 @@ groupwise_connect (CamelService *service, CamelException *ex)
 
 	d("in groupwise store connect\n");
 	
-	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL && 
-	     (service->status == CAMEL_SERVICE_DISCONNECTED)) 
-		return TRUE;
-	
+/*	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL || 
+	     (service->status == CAMEL_SERVICE_DISCONNECTED))
+		return FALSE; */
+	if (service->status == CAMEL_SERVICE_DISCONNECTED)
+		return FALSE;
+
+	if (!priv) {
+		store->priv = g_new0 (CamelGroupwiseStorePrivate, 1);
+		priv = store->priv;
+		camel_service_construct (service, service->session, service->provider, service->url, ex);
+	}
+
 	CAMEL_SERVICE_LOCK (service, connect_lock);
 	
 	if (priv->cnc) {
@@ -287,7 +294,9 @@ groupwise_connect (CamelService *service, CamelException *ex)
 		return FALSE;
 	}
 	
+	
 	service->status = CAMEL_SERVICE_CONNECTED;
+	((CamelOfflineStore *) store)->state = CAMEL_OFFLINE_STORE_NETWORK_AVAIL;
 
 	if (!e_gw_connection_get_version (priv->cnc)) {
 		camel_session_alert_user(session, 
@@ -323,19 +332,75 @@ groupwise_connect (CamelService *service, CamelException *ex)
 	return FALSE;
 
 }
+static void
+groupwise_disconnect_cleanup (CamelService *service, gboolean clean, CamelException *ex)
+{
+	CamelGroupwiseStore *groupwise_store = CAMEL_GROUPWISE_STORE (service);
+	CamelGroupwiseStorePrivate *priv = groupwise_store->priv;
+	
+	g_print ("camel_groupwise_store_finalize\n");
+	if (groupwise_store->summary) {
+		camel_store_summary_save ((CamelStoreSummary *)groupwise_store->summary);
+		camel_object_unref (groupwise_store->summary);
+	}
+	
+	if (priv) {
+		if (priv->user) {
+			g_free (priv->user);
+			priv->user = NULL;
+		}
+		if (priv->server_name) {
+			g_free (priv->server_name);
+			priv->server_name = NULL;
+		}
+		if (priv->port) {
+			g_free (priv->port);
+			priv->port = NULL;
+		}
+		if (priv->use_ssl) {
+			g_free (priv->use_ssl);
+			priv->use_ssl = NULL;
+		}
+		if (priv->base_url) {
+			g_free (priv->base_url);
+			priv->base_url = NULL;
+		}
+		
+		if (priv->storage_path)
+			g_free(priv->storage_path);
+
+		if(groupwise_store->root_container)
+			g_free (groupwise_store->root_container);
+		
+		if (priv->id_hash)
+			g_hash_table_destroy (priv->id_hash);
+
+		if (priv->name_hash)
+			g_hash_table_destroy (priv->name_hash);
+
+		if (priv->parent_hash)
+			g_hash_table_destroy (priv->parent_hash);
+
+		g_free (groupwise_store->priv);
+		groupwise_store->priv = NULL;
+	}
+}
 
 static gboolean
 groupwise_disconnect (CamelService *service, gboolean clean, CamelException *ex)
 {
 	CamelGroupwiseStore *groupwise_store = CAMEL_GROUPWISE_STORE (service);
 	
-	CAMEL_SERVICE_LOCK (groupwise_store, connect_lock);
-	if (groupwise_store->priv && groupwise_store->priv->cnc) {
-		g_object_unref (groupwise_store->priv->cnc);
-		groupwise_store->priv->cnc = NULL;
+	if (clean) {
+		CAMEL_SERVICE_LOCK (groupwise_store, connect_lock);
+		if (groupwise_store->priv && groupwise_store->priv->cnc) {
+			g_object_unref (groupwise_store->priv->cnc);
+			groupwise_store->priv->cnc = NULL;
+		}
+		CAMEL_SERVICE_UNLOCK (groupwise_store, connect_lock);
 	}
-	CAMEL_SERVICE_UNLOCK (groupwise_store, connect_lock);
 	
+	//groupwise_disconnect_cleanup (service, clean, ex);
 	return TRUE;
 }
 
@@ -396,7 +461,15 @@ groupwise_build_folder_info(CamelGroupwiseStore *gw_store, const char *parent_na
 		name = fi->full_name;
 	else
 		name++;
-	
+	if (!strcmp (folder_name, "Sent Items"))
+		fi->flags |= CAMEL_FOLDER_TYPE_SENT;
+	else if (!strcmp (folder_name, "Mailbox"))
+		fi->flags |= CAMEL_FOLDER_TYPE_INBOX;
+	else if (!strcmp (folder_name, "Trash"))
+		fi->flags |= CAMEL_FOLDER_TYPE_TRASH;
+	else if (!strcmp (folder_name, "Junk Mail"))
+		fi->flags |= CAMEL_FOLDER_TYPE_JUNK;
+		
 	fi->name = g_strdup(name);
 	return fi;
 }
@@ -484,7 +557,7 @@ groupwise_get_folder (CamelStore *store, const char *folder_name, guint32 flags,
 	char *container_id, *folder_dir, *storage_path;
 	EGwConnectionStatus status;
 	GList *list = NULL;
-	gboolean done = FALSE;
+	gboolean done = FALSE, all_ok = TRUE;
 	const char *position = E_GW_CURSOR_POSITION_END; 
 	int count = 0, cursor, summary_count = 0;
 	CamelStoreInfo *si = NULL;
@@ -533,19 +606,18 @@ groupwise_get_folder (CamelStore *store, const char *folder_name, guint32 flags,
 		return NULL;
 	}
 	g_free (folder_dir);
-
+	
 	si = camel_store_summary_path ((CamelStoreSummary *)gw_store->summary, folder_name);
 	if (si) {
-		camel_object_get (folder, NULL, CAMEL_FOLDER_TOTAL, &total, NULL);
+		total = si->total;
 		camel_store_summary_info_free ((CamelStoreSummary *)(gw_store)->summary, si);
-		g_print ("TOTAL:%d\n\n", total);
 	}
 
 	summary = (CamelGroupwiseSummary *) folder->summary;
 
 	summary_count = camel_folder_summary_count (folder->summary);
 	if(!summary_count || !summary->time_string) {
-		g_print ("\n\n** %s **: No summary as yet : using get cursor request\n\n", folder->name);
+		d(g_print ("\n\n** %s **: No summary as yet : using get cursor request\n\n", folder->name);)
 
 		status = e_gw_connection_create_cursor (priv->cnc, container_id, 
 				"peek id recipient attachments distribution subject status options priority startDate created delivered size",
@@ -560,6 +632,120 @@ groupwise_get_folder (CamelStore *store, const char *folder_name, guint32 flags,
 		camel_operation_start (NULL, _("Fetching summary information for new messages in %s"), folder->name);
 		camel_folder_summary_clear (folder->summary);
 
+		while (!done) {
+			status = e_gw_connection_read_cursor (priv->cnc, container_id, 
+							      cursor, FALSE, 
+							      CURSOR_ITEM_LIMIT, position, &list);
+			if (status != E_GW_CONNECTION_STATUS_OK) {
+				all_ok = FALSE;
+				break;
+				/*
+				CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+				e_gw_connection_destroy_cursor (priv->cnc, container_id, cursor);
+				//camel_folder_summary_clear (folder->summary);
+				camel_folder_summary_save (folder->summary);
+				camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_INVALID, _("Authentication failed"));
+				camel_operation_end (NULL);
+				camel_object_unref (folder);
+				g_free (container_id);
+				return NULL;*/
+			}
+			
+			count += g_list_length (list);
+		
+			if (total > 0)
+				camel_operation_progress (NULL, (100*count)/total);
+			gw_update_summary (folder, list,  ex);
+			
+			if (!list)
+				done = TRUE;
+			g_list_foreach (list, (GFunc)g_object_unref, NULL);
+			g_list_free (list);
+			list = NULL;
+			position = E_GW_CURSOR_POSITION_CURRENT;
+      		}
+
+		e_gw_connection_destroy_cursor (priv->cnc, container_id, cursor);
+
+		camel_operation_end (NULL);
+	} 
+
+	if (done && all_ok) {
+		if (summary->time_string)
+			g_free (summary->time_string);
+		summary->time_string = g_strdup (e_gw_connection_get_server_time (priv->cnc));
+	}
+
+	camel_folder_summary_save (folder->summary);
+
+	gw_store->current_folder = folder;
+	camel_object_ref (folder);
+
+	g_free (container_id);
+	CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+
+	return folder;
+}
+
+void 
+gw_store_reload_folder (CamelGroupwiseStore *gw_store, CamelFolder *folder, guint32 flags, CamelException *ex)
+{
+	CamelGroupwiseStorePrivate *priv = gw_store->priv;
+	CamelGroupwiseSummary *summary;
+	char *container_id;
+	EGwConnectionStatus status;
+	GList *list = NULL;
+	gboolean done = FALSE;
+	const char *position = E_GW_CURSOR_POSITION_END; 
+	int count = 0, cursor, summary_count = 0;
+	CamelStoreInfo *si = NULL;
+	guint total;
+	
+	camel_exception_clear (ex);
+
+	CAMEL_SERVICE_LOCK (gw_store, connect_lock);
+
+	if (!camel_groupwise_store_connected (gw_store, ex)) {
+		CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+		return;
+	}
+	
+	if (!E_IS_GW_CONNECTION( priv->cnc)) {
+		if (!groupwise_connect (CAMEL_SERVICE((CamelStore*)gw_store), ex)) {
+			camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE, _("Authentication failed"));
+			CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+			return;
+		}
+	}
+	
+	container_id = 	g_strdup (g_hash_table_lookup (priv->name_hash, folder->full_name));
+
+	si = camel_store_summary_path ((CamelStoreSummary *)gw_store->summary, folder->name);
+	if (si) {
+		total = si->total;
+		camel_store_summary_info_free ((CamelStoreSummary *)(gw_store)->summary, si);
+	}
+
+	summary = (CamelGroupwiseSummary *) folder->summary;
+	camel_folder_summary_clear (folder->summary);
+	camel_folder_summary_save (folder->summary);
+
+	summary_count = camel_folder_summary_count (folder->summary);
+	if(!summary_count || !summary->time_string) {
+		d(g_print ("\n\n** %s **: Summary missing???? Reloading summary....\n\n", folder->name);)
+
+		status = e_gw_connection_create_cursor (priv->cnc, container_id, 
+				"peek id recipient attachments distribution subject status options priority startDate created delivered size",
+				NULL,
+				&cursor);
+		if (status != E_GW_CONNECTION_STATUS_OK) {
+			CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
+			g_free (container_id);
+			return;
+		}
+
+		camel_operation_start (NULL, _("Fetching summary information for new messages in %s"), folder->name);
+
 		while (!done) {
 			status = e_gw_connection_read_cursor (priv->cnc, container_id, 
 							      cursor, FALSE, 
@@ -567,12 +753,11 @@ groupwise_get_folder (CamelStore *store, const char *folder_name, guint32 flags,
 			if (status != E_GW_CONNECTION_STATUS_OK) {
 				CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
 				e_gw_connection_destroy_cursor (priv->cnc, container_id, cursor);
-				camel_folder_summary_clear (folder->summary);
 				camel_folder_summary_save (folder->summary);
 				camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_INVALID, _("Authentication failed"));
 				camel_operation_end (NULL);
 				g_free (container_id);
-				return NULL;
+				return;
 			}
 			
 			count += g_list_length (list);
@@ -603,12 +788,10 @@ groupwise_get_folder (CamelStore *store, const char *folder_name, guint32 flags,
 	camel_folder_summary_save (folder->summary);
 
 	gw_store->current_folder = folder;
-	camel_object_ref (folder);
-
+	
 	g_free (container_id);
 	CAMEL_SERVICE_UNLOCK (gw_store, connect_lock);
-
-	return folder;
+	return;
 }
 
 CamelFolderInfo *
@@ -695,13 +878,16 @@ convert_to_folder_info (CamelGroupwiseStore *store, EGwContainer *container, con
 	if (e_gw_container_get_is_shared_by_me (container))
 		fi->flags |= CAMEL_FOLDER_SHARED_BY_ME;
 
-	if (type == E_GW_CONTAINER_TYPE_INBOX) {
+	/*if (type == E_GW_CONTAINER_TYPE_INBOX) {
 		fi->total = -1;
 		fi->unread = -1;
-	} else	if (type == E_GW_CONTAINER_TYPE_TRASH) {
+	} else	if (type == E_GW_CONTAINER_TYPE_TRASH) {*/
+			
+	if (type == E_GW_CONTAINER_TYPE_TRASH || type == E_GW_CONTAINER_TYPE_SENT) {
 		fi->total = e_gw_container_get_total_count (container);
-		fi->unread = 0;
-	}else {
+		fi->unread = 0; 
+		/* Done with a belief that user wont mark something as unread in Trash and Sent Items */
+	} else {
 		fi->total = e_gw_container_get_total_count (container);
 		fi->unread = e_gw_container_get_unread_count (container);
 	}
@@ -736,6 +922,13 @@ groupwise_folders_sync (CamelGroupwiseStore *store, CamelException *ex)
 	CamelStoreInfo *si = NULL;
 	int count, i;
 
+	if (!priv->cnc && ((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_AVAIL) {
+		if (((CamelService *)store)->status == CAMEL_SERVICE_DISCONNECTED){
+			((CamelService *)store)->status = CAMEL_SERVICE_CONNECTING;
+			groupwise_connect ((CamelService *)store, ex);
+		}
+	}
+
 	status = e_gw_connection_get_container_list (priv->cnc, "folders", &folder_list);
 	if (status != E_GW_CONNECTION_STATUS_OK) {
 		g_warning ("Could not get folder list..\n");
@@ -938,6 +1131,7 @@ groupwise_get_folder_info (CamelStore *store, const char *top, guint32 flags, Ca
 	if (top && groupwise_is_system_folder (top)) 
 		return groupwise_build_folder_info (groupwise_store, NULL, top );
 
+			
 	/*
 	 * Thanks to Michael, for his cached folders implementation in IMAP
 	 * is used as is here.
@@ -964,23 +1158,35 @@ groupwise_get_folder_info (CamelStore *store, const char *top, guint32 flags, Ca
 			}
 			CAMEL_SERVICE_UNLOCK (store, connect_lock);
 		}*/
+		if (groupwise_store->current_folder)
+			CAMEL_FOLDER_CLASS (CAMEL_OBJECT_GET_CLASS (groupwise_store->current_folder))->sync(groupwise_store->current_folder, FALSE, ex);
 		groupwise_folders_sync (groupwise_store, ex);
 	}
 
 	CAMEL_SERVICE_LOCK (store, connect_lock);
-	if ((groupwise_store->list_loaded == FALSE) && camel_groupwise_store_connected ((CamelGroupwiseStore *)store, ex)) {
-		groupwise_store->list_loaded = TRUE;
-		groupwise_folders_sync (groupwise_store, ex);
-		if (camel_exception_is_set (ex)) {
-			CAMEL_SERVICE_UNLOCK (store, connect_lock);
-			return NULL;
+	if ((groupwise_store->list_loaded == FALSE) && check_for_connection((CamelService *)store, ex)) {
+		if (!priv->cnc) {
+			if (groupwise_connect ((CamelService *)store, ex)) {
+				g_warning ("Could connect!!!\n");
+			} else 
+				g_warning ("Could not connect..failure connecting\n");
+		}
+		if (camel_groupwise_store_connected ((CamelGroupwiseStore *)store, ex)) {
+			groupwise_store->list_loaded = TRUE;
+			if (groupwise_store->current_folder)
+				CAMEL_FOLDER_CLASS (CAMEL_OBJECT_GET_CLASS (groupwise_store->current_folder))->sync(groupwise_store->current_folder, FALSE, ex);
+			groupwise_folders_sync (groupwise_store, ex);
+			if (camel_exception_is_set (ex)) {
+				CAMEL_SERVICE_UNLOCK (store, connect_lock);
+				return NULL;
+			}
+			camel_store_summary_touch ((CamelStoreSummary *)groupwise_store->summary);
+			camel_store_summary_save ((CamelStoreSummary *)groupwise_store->summary);
 		}
-		camel_store_summary_touch ((CamelStoreSummary *)groupwise_store->summary);
-		camel_store_summary_save ((CamelStoreSummary *)groupwise_store->summary);
 	}
 	CAMEL_SERVICE_UNLOCK (store, connect_lock);
 
-	//camel_exception_clear (ex);
+	/*camel_exception_clear (ex);*/
 	info = groupwise_get_folder_info_offline (store, top, flags, ex);
 	return info;
 }
@@ -1242,13 +1448,10 @@ gboolean
 camel_groupwise_store_connected (CamelGroupwiseStore *store, CamelException *ex)
 {
 	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_AVAIL
-	    && camel_service_connect ((CamelService *)store, ex))
+	    && camel_service_connect ((CamelService *)store, ex)) {
 		return TRUE;
-
-	if (!camel_exception_is_set (ex))
-		camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
-				_("You must be working online to complete this operation"));
-
+	}
+	/*Not online, so return FALSE*/
 	return FALSE;
 }
 
