@@ -58,6 +58,7 @@
 #include "camel-imap-command.h"
 #include "camel-imap-folder.h"
 #include "camel-imap-message-cache.h"
+#include "camel-imap-private.h"
 #include "camel-imap-store-summary.h"
 #include "camel-imap-store.h"
 #include "camel-imap-summary.h"
@@ -279,7 +280,7 @@ construct (CamelService *service, CamelSession *session,
 	imap_store->parameters = 0;
 	if (camel_url_get_param (url, "use_lsub"))
 		imap_store->parameters |= IMAP_PARAM_SUBSCRIPTIONS;
-	if (camel_url_get_param (url, "override_namespace") && camel_url_get_param (url, "namespace")) {
+	if (camel_url_get_param (url, "namespace")) {
 		imap_store->parameters |= IMAP_PARAM_OVERRIDE_NAMESPACE;
 		g_free(imap_store->namespace);
 		imap_store->namespace = g_strdup (camel_url_get_param (url, "namespace"));
@@ -977,11 +978,16 @@ connect_to_server_wrapper (CamelService *service, CamelException *ex)
 		camel_exception_clear (ex);
 		ai = camel_getaddrinfo(service->url->host, port, &hints, ex);
 	}
-	
 	if (ai == NULL)
 		return FALSE;
 	
-	ret = connect_to_server (service, ai, mode, ex);
+	if (!(ret = connect_to_server (service, ai, mode, ex)) && mode == MODE_SSL) {
+		camel_exception_clear (ex);
+		ret = connect_to_server (service, ai, MODE_TLS, ex);
+	} else if (!ret && mode == MODE_TLS) {
+		camel_exception_clear (ex);
+		ret = connect_to_server (service, ai, MODE_CLEAR, ex);
+	}
 	
 	camel_freeaddrinfo (ai);
 	
@@ -2617,10 +2623,7 @@ refresh_refresh(CamelSession *session, CamelSessionThreadMsg *msg)
 
 	if (store->namespace && store->namespace[0]) {
 		char *pattern;
-		
-		get_folders_sync(store, "INBOX", &m->ex);
-		if (camel_exception_is_set(&m->ex))
-			goto done;
+
 		get_folders_sync(store, store->namespace, &m->ex);
 		if (camel_exception_is_set(&m->ex))
 			goto done;
@@ -2670,7 +2673,7 @@ get_folder_info_online (CamelStore *store, const char *top, guint32 flags, Camel
 		time_t now;
 		int ref;
 
-		now = time(NULL);
+		now = time(0);
 		ref = now > imap_store->refresh_stamp+60*60*1;
 		if (ref) {
 			CAMEL_SERVICE_LOCK(store, connect_lock);
@@ -2699,10 +2702,6 @@ get_folder_info_online (CamelStore *store, const char *top, guint32 flags, Camel
 
 		if (top[0] == 0) {
 			if (imap_store->namespace && imap_store->namespace[0]) {
-				get_folders_sync(imap_store, "INBOX", ex);
-				if (camel_exception_is_set(ex))
-					goto fail;
-				
 				i = strlen(imap_store->namespace)-1;
 				pattern = g_alloca(i+5);
 				strcpy(pattern, imap_store->namespace);
@@ -2717,7 +2716,7 @@ get_folder_info_online (CamelStore *store, const char *top, guint32 flags, Camel
 			}
 		} else {
 			char *name;
-			
+
 			name = camel_imap_store_summary_full_from_path(imap_store->summary, top);
 			if (name == NULL)
 				name = camel_imap_store_summary_path_to_full(imap_store->summary, top, imap_store->dir_sep);
@@ -2754,7 +2753,6 @@ get_folder_info_offline (CamelStore *store, const char *top,
 			 guint32 flags, CamelException *ex)
 {
 	CamelImapStore *imap_store = CAMEL_IMAP_STORE (store);
-	gboolean include_inbox = FALSE;
 	CamelFolderInfo *fi;
 	GPtrArray *folders;
 	char *pattern, *name;
@@ -2767,14 +2765,12 @@ get_folder_info_offline (CamelStore *store, const char *top,
 
 	folders = g_ptr_array_new ();
 
-	if (top == NULL || top[0] == '\0') {
-		include_inbox = TRUE;
+	if (top == NULL)
 		top = "";
-	}
 
 	/* get starting point */
 	if (top[0] == 0) {
-		if (imap_store->namespace && imap_store->namespace[0]) {
+		if (imap_store->namespace) {
 			name = g_strdup(imap_store->summary->namespace->full_name);
 			top = imap_store->summary->namespace->path;
 		} else
@@ -2798,10 +2794,9 @@ get_folder_info_offline (CamelStore *store, const char *top,
 
 		if (si == NULL)
 			continue;
-		
+
 		if ((!strcmp(name, camel_imap_store_info_full_name(imap_store->summary, si))
-		     || imap_match_pattern(imap_store->dir_sep, pattern, camel_imap_store_info_full_name(imap_store->summary, si))
-		     || (include_inbox && !g_ascii_strcasecmp (camel_imap_store_info_full_name(imap_store->summary, si), "INBOX")))
+		     || imap_match_pattern(imap_store->dir_sep, pattern, camel_imap_store_info_full_name(imap_store->summary, si)))
 		    && ((imap_store->parameters & IMAP_PARAM_SUBSCRIPTIONS) == 0
 			|| (flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIBED) == 0
 			|| (si->flags & CAMEL_STORE_INFO_FOLDER_SUBSCRIBED))) {
@@ -2814,11 +2809,11 @@ get_folder_info_offline (CamelStore *store, const char *top,
 			   it.  See create folder */
 			if (fi->flags & CAMEL_FOLDER_NOINFERIORS)
 				fi->flags = (fi->flags & ~CAMEL_FOLDER_NOINFERIORS) | CAMEL_FOLDER_NOCHILDREN;
-			
+
 			/* blah, this gets lost somewhere, i can't be bothered finding out why */
 			if (!g_ascii_strcasecmp(fi->full_name, "inbox"))
 				fi->flags = (fi->flags & ~CAMEL_FOLDER_TYPE_MASK) | CAMEL_FOLDER_TYPE_INBOX;
-			
+
 			if (si->flags & CAMEL_FOLDER_NOSELECT) {
 				CamelURL *url = camel_url_new(fi->uri, NULL);
 				
