@@ -45,7 +45,7 @@ add_folder_esource (ExchangeAccount *account,
 	GConfClient *client;
 	gboolean is_contacts_folder = TRUE, group_new = FALSE, source_new = FALSE;
 	const char *offline = NULL;
-	char *username, *authtype;
+	char *username, *authtype = NULL;
 	int mode;
 	ESourceList *source_list = NULL;
 	gboolean offline_flag;
@@ -88,11 +88,14 @@ add_folder_esource (ExchangeAccount *account,
         		g_free (relative_uri);
 			return;
 		}
-		if (is_contacts_folder && g_str_has_prefix (physical_uri, "gal://"))
+		if (is_contacts_folder && g_str_has_prefix (physical_uri, "gal://")) {
 			source = e_source_new_with_absolute_uri (folder_name,
 								 physical_uri);
-		else
+			e_source_set_property (source, "completion", "true");
+		}
+		else {
 			source = e_source_new (folder_name, relative_uri);
+		}
 
 		if (mode == OFFLINE_MODE) {
 			/* If account is marked for offline sync during account
@@ -117,11 +120,14 @@ add_folder_esource (ExchangeAccount *account,
 		if((source = e_source_group_peek_source_by_name (source_group, 
 							folder_name)) == NULL) {
 			printf("old group, new source\n");
-			if (is_contacts_folder && g_str_has_prefix (physical_uri, "gal://"))
+			if (is_contacts_folder && g_str_has_prefix (physical_uri, "gal://")) {
 				source = e_source_new_with_absolute_uri (
 						folder_name, physical_uri);
-			else
+				e_source_set_property (source, "completion", "true");
+			}
+			else {
         			source = e_source_new (folder_name, relative_uri);
+			}
 
 			if (mode == OFFLINE_MODE)
 				e_source_set_property (source, "offline_sync", "1");
@@ -142,10 +148,11 @@ add_folder_esource (ExchangeAccount *account,
 			offline = e_source_get_property (source, "offline_sync");
 			if (!offline) {
 				/* Folder doesn't have any offline property set */
-				if (mode == OFFLINE_MODE) 
+				if (mode == OFFLINE_MODE) {
 					e_source_set_property (source, "offline_sync", "1");
+					e_source_list_sync (source_list, NULL);
+				}
 			}
-			e_source_list_sync (source_list, NULL);
 		}
 	}
 
@@ -181,6 +188,7 @@ add_folder_esource (ExchangeAccount *account,
 	}
 
 	g_free (relative_uri);
+	g_free (authtype);
 
 	if (source_new) 
 		g_object_unref (source);
@@ -321,5 +329,6 @@ is_offline (void)
 		offline = gconf_value_get_bool (value);
 
 	g_object_unref (client);
+	gconf_value_free (value);
 	return offline;
 }
